#!/usr/bin/env python
import os
import sys

import django


def runtests(*test_args):
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "account.tests.settings")
    django.setup()

    parent = os.path.dirname(os.path.abspath(__file__))
    sys.path.insert(0, parent)

    from django.test.runner import DiscoverRunner
    runner_class = DiscoverRunner
    if not test_args:
        test_args = ["account/tests"]

    failures = runner_class(verbosity=1, interactive=True, failfast=False).run_tests(test_args)
    sys.exit(failures)


if __name__ == "__main__":
    runtests(*sys.argv[1:])



from django.contrib.auth.models import User
from django.forms import ValidationError
from django.test import TestCase, override_settings

from account.models import EmailAddress


@override_settings(ACCOUNT_EMAIL_UNIQUE=True)
class UniqueEmailAddressTestCase(TestCase):
    def test_unique_email(self):
        user = User.objects.create_user("user1", email="user1@example.com", password="password")

        email_1 = EmailAddress(user=user, email="user2@example.com")
        email_1.full_clean()
        email_1.save()

        validation_error = False
        try:
            email_2 = EmailAddress(user=user, email="USER2@example.com")
            email_2.full_clean()
            email_2.save()
        except ValidationError:
            validation_error = True

        self.assertTrue(validation_error)



import os
import sys


extensions = []
templates_path = []
source_suffix = ".rst"
master_doc = "index"
project = "django-user-accounts"
copyright_holder = "James Tauber and contributors"
copyright = "2014, {0}".format(copyright_holder)
exclude_patterns = ["_build"]
pygments_style = "sphinx"
html_theme = "default"
htmlhelp_basename = "{0}doc".format(project)
latex_documents = [
  ("index", "{0}.tex".format(project), "{0} Documentation".format(project),
   "Pinax", "manual"),
]
man_pages = [
    ("index", project, "{0} Documentation".format(project),
     ["Pinax"], 1)
]

sys.path.insert(0, os.pardir)
m = __import__("account")

version = m.__version__
release = version



import importlib

from django.conf import settings  # noqa
from django.core.exceptions import ImproperlyConfigured

from account.languages import LANGUAGES
from account.timezones import TIMEZONES
from appconf import AppConf


def load_path_attr(path):
    i = path.rfind(".")
    module, attr = path[:i], path[i + 1:]
    try:
        mod = importlib.import_module(module)
    except ImportError as e:
        raise ImproperlyConfigured("Error importing {0}: '{1}'".format(module, e))
    try:
        attr = getattr(mod, attr)
    except AttributeError:
        raise ImproperlyConfigured("Module '{0}' does not define a '{1}'".format(module, attr))
    return attr


class AccountAppConf(AppConf):

    OPEN_SIGNUP = True
    LOGIN_URL = "account_login"
    LOGOUT_URL = "account_logout"
    SIGNUP_REDIRECT_URL = "/"
    LOGIN_REDIRECT_URL = "/"
    LOGOUT_REDIRECT_URL = "/"
    PASSWORD_CHANGE_REDIRECT_URL = "account_password"
    PASSWORD_RESET_REDIRECT_URL = "account_login"
    PASSWORD_RESET_TOKEN_URL = "account_password_reset_token"
    PASSWORD_EXPIRY = 0
    PASSWORD_USE_HISTORY = False
    PASSWORD_STRIP = True
    REMEMBER_ME_EXPIRY = 60 * 60 * 24 * 365 * 10
    USER_DISPLAY = lambda user: user.username  # noqa
    CREATE_ON_SAVE = True
    EMAIL_UNIQUE = True
    EMAIL_CONFIRMATION_REQUIRED = False
    EMAIL_CONFIRMATION_EMAIL = True
    EMAIL_CONFIRMATION_EXPIRE_DAYS = 3
    EMAIL_CONFIRMATION_AUTO_LOGIN = False
    EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL = "account_login"
    EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL = None
    EMAIL_CONFIRMATION_URL = "account_confirm_email"
    SETTINGS_REDIRECT_URL = "account_settings"
    NOTIFY_ON_PASSWORD_CHANGE = True
    DELETION_EXPUNGE_HOURS = 48
    DEFAULT_HTTP_PROTOCOL = "https"
    HOOKSET = "account.hooks.AccountDefaultHookSet"
    TIMEZONES = TIMEZONES
    LANGUAGES = LANGUAGES

    def configure_hookset(self, value):
        return load_path_attr(value)()



from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.db.models import Q

from account.models import EmailAddress
from account.utils import get_user_lookup_kwargs

User = get_user_model()


class AccountModelBackend(ModelBackend):
    """
    This authentication backend ensures that the account is always selected
    on any query with the user, so we don't issue extra unnecessary queries
    """

    def get_user(self, user_id):
        """Get the user and select account at the same time"""
        user = User._default_manager.filter(pk=user_id).select_related("account").first()
        if not user:
            return None
        return user if self.user_can_authenticate(user) else None


class UsernameAuthenticationBackend(AccountModelBackend):
    """Username authentication"""

    def authenticate(self, request, username=None, password=None, **kwargs):
        """Authenticate the user based on user"""
        if username is None or password is None:
            return None

        try:
            lookup_kwargs = get_user_lookup_kwargs({
                "{username}__iexact": username
            })
            user = User.objects.get(**lookup_kwargs)
        except User.DoesNotExist:
            return None

        if user.check_password(password):
            return user


class EmailAuthenticationBackend(AccountModelBackend):
    """Email authentication"""

    def authenticate(self, request, username=None, password=None, **kwargs):
        """Authenticate the user based email"""
        qs = EmailAddress.objects.filter(Q(primary=True) | Q(verified=True))

        if username is None or password is None:
            return None

        try:
            email_address = qs.get(email__iexact=username)
        except EmailAddress.DoesNotExist:
            return None

        user = email_address.user
        if user.check_password(password):
            return user



import django.dispatch

user_signed_up = django.dispatch.Signal()
user_sign_up_attempt = django.dispatch.Signal()
user_logged_in = django.dispatch.Signal()
user_login_attempt = django.dispatch.Signal()
signup_code_sent = django.dispatch.Signal()
signup_code_used = django.dispatch.Signal()
email_confirmed = django.dispatch.Signal()
email_confirmation_sent = django.dispatch.Signal()
password_changed = django.dispatch.Signal()
password_expired = django.dispatch.Signal()



import datetime

import django
from django.contrib.auth.hashers import check_password, make_password
from django.contrib.auth.models import User
from django.test import TestCase, modify_settings, override_settings
from django.urls import reverse

import pytz

from account.models import PasswordExpiry, PasswordHistory
from account.utils import check_password_expired


def middleware_kwarg(value):
    if django.VERSION >= (1, 10):
        kwarg = "MIDDLEWARE"
    else:
        kwarg = "MIDDLEWARE_CLASSES"
    return {kwarg: value}


@override_settings(
    ACCOUNT_PASSWORD_USE_HISTORY=True
)
@modify_settings(
    **middleware_kwarg({
        "append": "account.middleware.ExpiredPasswordMiddleware"
    })
)
class PasswordExpirationTestCase(TestCase):

    def setUp(self):
        self.username = "user1"
        self.email = "user1@example.com"
        self.password = "changeme"
        self.user = User.objects.create_user(
            self.username,
            email=self.email,
            password=self.password,
        )
        # create PasswordExpiry for user
        self.expiry = PasswordExpiry.objects.create(
            user=self.user,
            expiry=60,  # password expires after sixty seconds
        )
        # create PasswordHistory for user
        self.history = PasswordHistory.objects.create(
            user=self.user,
            password=make_password(self.password)
        )

    def test_signup(self):
        """
        Ensure new user has one PasswordHistory and no PasswordExpiry.
        """
        email = "foobar@example.com"
        password = "bar"
        post_data = {
            "username": "foo",
            "password": password,
            "password_confirm": password,
            "email": email,
        }
        response = self.client.post(reverse("account_signup"), post_data)
        self.assertEqual(response.status_code, 302)
        user = User.objects.get(email=email)
        self.assertFalse(hasattr(user, "password_expiry"))
        latest_history = user.password_history.latest("timestamp")
        self.assertTrue(latest_history)

        # verify password is not expired
        self.assertFalse(check_password_expired(user))
        # verify raw password matches encrypted password in history
        self.assertTrue(check_password(password, latest_history.password))

    def test_get_not_expired(self):
        """
        Ensure authenticated user can retrieve account settings page
        without "password change" redirect.
        """
        self.client.login(username=self.username, password=self.password)

        # get account settings page (could be any application page)
        response = self.client.get(reverse("account_settings"))
        self.assertEqual(response.status_code, 200)

    def test_get_expired(self):
        """
        Ensure authenticated user is redirected to change password
        when retrieving account settings page if password is expired.
        """
        # set PasswordHistory timestamp in past so password is expired.
        self.history.timestamp = (
            datetime.datetime.now(tz=pytz.UTC) - datetime.timedelta(days=1, seconds=self.expiry.expiry)
        )
        self.history.save()

        self.client.login(username=self.username, password=self.password)

        # get account settings page (could be any application page)
        url_name = "account_settings"
        response = self.client.get(reverse(url_name))

        # verify desired page is set as "?next=" in redirect URL
        redirect_url = "{}?next={}".format(reverse("account_password"), url_name)
        self.assertRedirects(response, redirect_url)

    def test_password_expiration_reset(self):
        """
        Ensure changing password results in new PasswordHistory.
        """
        history_count = self.user.password_history.count()

        # get login
        self.client.login(username=self.username, password=self.password)

        # post new password to reset PasswordHistory
        new_password = "lynyrdskynyrd"
        post_data = {
            "password_current": self.password,
            "password_new": new_password,
            "password_new_confirm": new_password,
        }
        self.client.post(
            reverse("account_password"),
            post_data
        )
        # Should see one more history entry for this user
        self.assertEqual(self.user.password_history.count(), history_count + 1)

        latest = PasswordHistory.objects.latest("timestamp")
        self.assertTrue(latest != self.history)
        self.assertTrue(latest.timestamp > self.history.timestamp)


@modify_settings(
    **middleware_kwarg({
        "append": "account.middleware.ExpiredPasswordMiddleware"
    })
)
class ExistingUserNoHistoryTestCase(TestCase):
    """
    Tests where user has no PasswordHistory.
    """

    def setUp(self):
        self.username = "user1"
        self.email = "user1@example.com"
        self.password = "changeme"
        self.user = User.objects.create_user(
            self.username,
            email=self.email,
            password=self.password,
        )

    def test_get_no_history(self):
        """
        Ensure authenticated user without password history can retrieve
        account settings page without "password change" redirect.
        """
        self.client.login(username=self.username, password=self.password)

        with override_settings(
            ACCOUNT_PASSWORD_USE_HISTORY=True
        ):
            # get account settings page (could be any application page)
            response = self.client.get(reverse("account_settings"))
            self.assertEqual(response.status_code, 200)

    def test_password_expiration_reset(self):
        """
        Ensure changing password results in new PasswordHistory,
        even when no PasswordHistory exists.
        """
        history_count = self.user.password_history.count()

        # get login
        self.client.login(username=self.username, password=self.password)

        # post new password to reset PasswordHistory
        new_password = "lynyrdskynyrd"
        post_data = {
            "password_current": self.password,
            "password_new": new_password,
            "password_new_confirm": new_password,
        }
        with override_settings(
            ACCOUNT_PASSWORD_USE_HISTORY=True
        ):
            self.client.post(
                reverse("account_password"),
                post_data
            )
            # Should see one more history entry for this user
            self.assertEqual(self.user.password_history.count(), history_count + 1)

    def test_password_reset(self):
        """
        Ensure changing password results in NO new PasswordHistory
        when ACCOUNT_PASSWORD_USE_HISTORY == False.
        """
        # get login
        self.client.login(username=self.username, password=self.password)

        # post new password to reset PasswordHistory
        new_password = "lynyrdskynyrd"
        post_data = {
            "password_current": self.password,
            "password_new": new_password,
            "password_new_confirm": new_password,
        }
        with override_settings(
            ACCOUNT_PASSWORD_USE_HISTORY=False
        ):
            self.client.post(
                reverse("account_password"),
                post_data
            )
            # history count should be zero
            self.assertEqual(self.user.password_history.count(), 0)



from django.contrib.auth import authenticate
from django.contrib.auth.models import User
from django.test import TestCase, override_settings


@override_settings(
    AUTHENTICATION_BACKENDS=[
        "account.auth_backends.UsernameAuthenticationBackend"
    ]
)
class UsernameAuthenticationBackendTestCase(TestCase):

    def create_user(self, username, email, password):
        user = User.objects.create_user(username, email=email, password=password)
        return user

    def test_successful_auth(self):
        created_user = self.create_user("user1", "user1@example.com", "password")
        authed_user = authenticate(username="user1", password="password")
        self.assertTrue(authed_user is not None)
        self.assertEqual(created_user.pk, authed_user.pk)

    def test_unsuccessful_auth(self):
        authed_user = authenticate(username="user-does-not-exist", password="password")
        self.assertTrue(authed_user is None)

    def test_missing_credentials(self):
        self.create_user("user1", "user1@example.com", "password")
        self.assertTrue(authenticate() is None)
        self.assertTrue(authenticate(username="user1") is None)

    def test_successful_auth_django_2_1(self):
        created_user = self.create_user("user1", "user1@example.com", "password")
        request = None
        authed_user = authenticate(request, username="user1", password="password")
        self.assertTrue(authed_user is not None)
        self.assertEqual(created_user.pk, authed_user.pk)

    def test_unsuccessful_auth_django_2_1(self):
        request = None
        authed_user = authenticate(request, username="user-does-not-exist", password="password")
        self.assertTrue(authed_user is None)


@override_settings(
    AUTHENTICATION_BACKENDS=[
        "account.auth_backends.EmailAuthenticationBackend"
    ]
)
class EmailAuthenticationBackendTestCase(TestCase):

    def create_user(self, username, email, password):
        user = User.objects.create_user(username, email=email, password=password)
        return user

    def test_successful_auth(self):
        created_user = self.create_user("user1", "user1@example.com", "password")
        authed_user = authenticate(username="user1@example.com", password="password")
        self.assertTrue(authed_user is not None)
        self.assertEqual(created_user.pk, authed_user.pk)

    def test_unsuccessful_auth(self):
        authed_user = authenticate(username="user-does-not-exist", password="password")
        self.assertTrue(authed_user is None)

    def test_missing_credentials(self):
        self.create_user("user1", "user1@example.com", "password")
        self.assertTrue(authenticate() is None)
        self.assertTrue(authenticate(username="user1@example.com") is None)

    def test_successful_auth_django_2_1(self):
        created_user = self.create_user("user1", "user1@example.com", "password")
        request = None
        authed_user = authenticate(request, username="user1@example.com", password="password")
        self.assertTrue(authed_user is not None)
        self.assertEqual(created_user.pk, authed_user.pk)

    def test_unsuccessful_auth_django_2_1(self):
        request = None
        authed_user = authenticate(request, username="user-does-not-exist", password="password")
        self.assertTrue(authed_user is None)



import datetime

from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand

import pytz
from account.models import PasswordHistory


class Command(BaseCommand):

    help = "Create password history for all users without existing history."

    def add_arguments(self, parser):
        parser.add_argument(
            "-d", "--days",
            type=int,
            nargs="?",
            default=10,
            help="age of current password (in days)"
        )
        parser.add_argument(
            "-f", "--force",
            action="store_true",
            help="create new password history for all users, regardless of existing history"
        )

    def handle(self, *args, **options):
        User = get_user_model()
        users = User.objects.all()
        if not options["force"]:
            users = users.filter(password_history=None)

        if not users:
            return "No users found without password history"

        days = options["days"]
        timestamp = datetime.datetime.now(tz=pytz.UTC) - datetime.timedelta(days=days)

        # Create new PasswordHistory on `timestamp`
        PasswordHistory.objects.bulk_create(
            [PasswordHistory(user=user, timestamp=timestamp) for user in users]
        )

        return "Password history set to {} for {} users".format(timestamp, len(users))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-13 08:55
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('account', '0002_fix_str'),
    ]

    operations = [
        migrations.CreateModel(
            name='PasswordExpiry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('expiry', models.PositiveIntegerField(default=0)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='password_expiry', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
        ),
        migrations.CreateModel(
            name='PasswordHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=255)),
                ('timestamp', models.DateTimeField(default=django.utils.timezone.now)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='password_history', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.test import TestCase

from account.models import SignupCode


class SignupCodeModelTestCase(TestCase):
    def test_exists_no_match(self):
        code = SignupCode(email="foobar@example.com", code="FOOFOO")
        code.save()

        self.assertFalse(SignupCode.exists(code="BARBAR"))
        self.assertFalse(SignupCode.exists(email="bar@example.com"))
        self.assertFalse(SignupCode.exists(email="bar@example.com", code="BARBAR"))
        self.assertFalse(SignupCode.exists())

    def test_exists_email_only_match(self):
        code = SignupCode(email="foobar@example.com", code="FOOFOO")
        code.save()

        self.assertTrue(SignupCode.exists(email="foobar@example.com"))

    def test_exists_code_only_match(self):
        code = SignupCode(email="foobar@example.com", code="FOOFOO")
        code.save()

        self.assertTrue(SignupCode.exists(code="FOOFOO"))
        self.assertTrue(SignupCode.exists(email="bar@example.com", code="FOOFOO"))

    def test_exists_email_match_code_mismatch(self):
        code = SignupCode(email="foobar@example.com", code="FOOFOO")
        code.save()

        self.assertTrue(SignupCode.exists(email="foobar@example.com", code="BARBAR"))

    def test_exists_code_match_email_mismatch(self):
        code = SignupCode(email="foobar@example.com", code="FOOFOO")
        code.save()

        self.assertTrue(SignupCode.exists(email="bar@example.com", code="FOOFOO"))

    def test_exists_both_match(self):
        code = SignupCode(email="foobar@example.com", code="FOOFOO")
        code.save()

        self.assertTrue(SignupCode.exists(email="foobar@example.com", code="FOOFOO"))



from django.contrib import admin

from account.models import (
    Account,
    AccountDeletion,
    EmailAddress,
    PasswordExpiry,
    PasswordHistory,
    SignupCode,
)


class SignupCodeAdmin(admin.ModelAdmin):

    list_display = ["code", "max_uses", "use_count", "expiry", "created"]
    search_fields = ["code", "email"]
    list_filter = ["created"]
    raw_id_fields = ["inviter"]


class AccountAdmin(admin.ModelAdmin):

    raw_id_fields = ["user"]

    def get_queryset(self, request):
        return super().get_queryset(request).select_related('user')


class AccountDeletionAdmin(AccountAdmin):

    list_display = ["email", "date_requested", "date_expunged"]


class EmailAddressAdmin(AccountAdmin):

    list_display = ["user", "email", "verified", "primary"]
    search_fields = ["email", "user__username"]

    def get_queryset(self, request):
        return super().get_queryset(request).select_related('user')


class PasswordExpiryAdmin(admin.ModelAdmin):

    raw_id_fields = ["user"]


class PasswordHistoryAdmin(admin.ModelAdmin):

    raw_id_fields = ["user"]
    list_display = ["user", "timestamp"]
    list_filter = ["user"]
    ordering = ["user__username", "-timestamp"]

    def get_queryset(self, request):
        return super().get_queryset(request).select_related('user')


admin.site.register(Account, AccountAdmin)
admin.site.register(SignupCode, SignupCodeAdmin)
admin.site.register(AccountDeletion, AccountDeletionAdmin)
admin.site.register(EmailAddress, EmailAddressAdmin)
admin.site.register(PasswordExpiry, PasswordExpiryAdmin)
admin.site.register(PasswordHistory, PasswordHistoryAdmin)



#!/usr/bin/env python
import os
import sys

import django
from django.conf import settings

DEFAULT_SETTINGS = dict(
    INSTALLED_APPS=[
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sites",
        "account",
        "account.tests"
    ],
    MIDDLEWARE_CLASSES=[],
    DATABASES={
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": ":memory:",
        }
    },
    SITE_ID=1,
    ROOT_URLCONF="account.tests.urls",
    SECRET_KEY="notasecret",
)


def run(*args):
    if not settings.configured:
        settings.configure(**DEFAULT_SETTINGS)

    django.setup()

    parent = os.path.dirname(os.path.abspath(__file__))
    sys.path.insert(0, parent)

    django.core.management.call_command(
        "makemigrations",
        "account",
        *args
    )


if __name__ == "__main__":
    run(*sys.argv[1:])



from urllib.parse import urlparse

from django.conf import settings
from django.contrib.auth.models import User
from django.core import mail
from django.test import TestCase, override_settings
from django.urls import reverse
from django.utils.http import int_to_base36

from account.models import EmailConfirmation, SignupCode
from account.views import INTERNAL_RESET_URL_TOKEN, PasswordResetTokenView


class SignupViewTestCase(TestCase):

    def test_get(self):
        response = self.client.get(reverse("account_signup"))
        self.assertEqual(response.status_code, 200)

    def test_post(self):
        data = {
            "username": "foo",
            "password": "bar",
            "password_confirm": "bar",
            "email": "foobar@example.com",
        }
        response = self.client.post(reverse("account_signup"), data)
        self.assertEqual(response.status_code, 302)

    def test_closed(self):
        with self.settings(ACCOUNT_OPEN_SIGNUP=False):
            data = {
                "username": "foo",
                "password": "bar",
                "password_confirm": "bar",
                "email": "foobar@example.com",
            }
            response = self.client.post(reverse("account_signup"), data)
            self.assertEqual(response.status_code, 200)
            self.assertEqual(response.template_name, "account/signup_closed.html")

    def test_valid_code(self):
        signup_code = SignupCode.create()
        signup_code.save()
        with self.settings(ACCOUNT_OPEN_SIGNUP=False):
            data = {
                "username": "foo",
                "password": "bar",
                "password_confirm": "bar",
                "email": "foobar@example.com",
                "code": signup_code.code,
            }
            response = self.client.post(reverse("account_signup"), data)
            self.assertEqual(response.status_code, 302)

    def test_invalid_code(self):
        with self.settings(ACCOUNT_OPEN_SIGNUP=False):
            data = {
                "username": "foo",
                "password": "bar",
                "password_confirm": "bar",
                "email": "foobar@example.com",
                "code": "abc123",
            }
            response = self.client.post(reverse("account_signup"), data)
            self.assertEqual(response.status_code, 200)
            self.assertEqual(response.template_name, "account/signup_closed.html")

    def test_get_authenticated(self):
        User.objects.create_user("foo", password="bar")
        self.client.login(username="foo", password="bar")

        with self.settings(ACCOUNT_LOGIN_REDIRECT_URL="/logged-in/"):
            response = self.client.get(reverse("account_signup"))
            self.assertRedirects(response, "/logged-in/", fetch_redirect_response=False)

    def test_post_authenticated(self):
        User.objects.create_user("foo", password="bar")
        self.client.login(username="foo", password="bar")

        with self.settings(ACCOUNT_LOGIN_REDIRECT_URL="/logged-in/"):
            data = {
                "username": "foo",
                "password": "bar",
                "password_confirm": "bar",
                "email": "foobar@example.com",
                "code": "abc123",
            }
            response = self.client.post(reverse("account_signup"), data)
            self.assertEqual(response.status_code, 404)

    def test_get_next_url(self):
        next_url = "/next-url/"
        data = {
            "username": "foo",
            "password": "bar",
            "password_confirm": "bar",
            "email": "foobar@example.com",
        }
        response = self.client.post("{}?next={}".format(reverse("account_signup"), next_url), data)
        self.assertRedirects(response, next_url, fetch_redirect_response=False)

    def test_post_next_url(self):
        next_url = "/next-url/"
        data = {
            "username": "foo",
            "password": "bar",
            "password_confirm": "bar",
            "email": "foobar@example.com",
            "next": next_url,
        }
        response = self.client.post(reverse("account_signup"), data)
        self.assertRedirects(response, next_url, fetch_redirect_response=False)

    def test_session_next_url(self):
        next_url = "/next-url/"
        session = self.client.session
        session["redirect_to"] = next_url
        session.save()
        data = {
            "username": "foo",
            "password": "bar",
            "password_confirm": "bar",
            "email": "foobar@example.com",
        }
        response = self.client.post(reverse("account_signup"), data)
        self.assertRedirects(response, next_url, fetch_redirect_response=False)


class LoginViewTestCase(TestCase):

    def signup(self):
        data = {
            "username": "foo",
            "password": "bar",
            "password_confirm": "bar",
            "email": "foobar@example.com",
            "code": "abc123",
        }
        self.client.post(reverse("account_signup"), data)
        self.client.logout()

    def test_get(self):
        response = self.client.get(reverse("account_login"))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name, ["account/login.html"])

    def test_post_empty(self):
        data = {}
        response = self.client.post(reverse("account_login"), data)
        self.assertEqual(response.status_code, 200)
        self.assertFalse(response.context["form"].is_valid())

    @override_settings(
        AUTHENTICATION_BACKENDS=[
            "account.auth_backends.UsernameAuthenticationBackend",
        ]
    )
    def test_post_success(self):
        self.signup()
        data = {
            "username": "foo",
            "password": "bar",
        }
        response = self.client.post(reverse("account_login"), data)
        self.assertRedirects(
            response,
            settings.ACCOUNT_LOGIN_REDIRECT_URL,
            fetch_redirect_response=False
        )


class LogoutViewTestCase(TestCase):

    def signup(self):
        data = {
            "username": "foo",
            "password": "bar",
            "password_confirm": "bar",
            "email": "foobar@example.com",
            "code": "abc123",
        }
        self.client.post(reverse("account_signup"), data)

    def test_get_anonymous(self):
        response = self.client.get(reverse("account_logout"))
        self.assertRedirects(
            response,
            settings.ACCOUNT_LOGOUT_REDIRECT_URL,
            fetch_redirect_response=False
        )

    def test_get_authenticated(self):
        self.signup()
        response = self.client.get(reverse("account_logout"))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name, ["account/logout.html"])

    def test_post_anonymous(self):
        response = self.client.post(reverse("account_logout"), {})
        self.assertRedirects(
            response,
            settings.ACCOUNT_LOGOUT_REDIRECT_URL,
            fetch_redirect_response=False
        )

    def test_post_authenticated(self):
        self.signup()
        response = self.client.post(reverse("account_logout"), {})
        self.assertRedirects(
            response,
            settings.ACCOUNT_LOGOUT_REDIRECT_URL,
            fetch_redirect_response=False
        )


class ConfirmEmailViewTestCase(TestCase):

    def signup(self):
        data = {
            "username": "foo",
            "password": "bar",
            "password_confirm": "bar",
            "email": "foobar@example.com",
            "code": "abc123",
        }
        self.client.post(reverse("account_signup"), data)
        return EmailConfirmation.objects.get()

    def test_get_good_key(self):
        email_confirmation = self.signup()
        response = self.client.get(reverse("account_confirm_email", kwargs={"key": email_confirmation.key}))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name, ["account/email_confirm.html"])

    def test_get_bad_key(self):
        response = self.client.get(reverse("account_confirm_email", kwargs={"key": "badkey"}))
        self.assertEqual(response.status_code, 404)

    @override_settings(ACCOUNT_EMAIL_CONFIRMATION_REQUIRED=True)
    def test_post_required(self):
        email_confirmation = self.signup()
        response = self.client.post(reverse("account_confirm_email", kwargs={"key": email_confirmation.key}), {})
        self.assertRedirects(
            response,
            reverse(settings.ACCOUNT_EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL),
            fetch_redirect_response=False
        )

    @override_settings(ACCOUNT_EMAIL_CONFIRMATION_REQUIRED=False)
    def test_post_not_required(self):
        email_confirmation = self.signup()
        response = self.client.post(reverse("account_confirm_email", kwargs={"key": email_confirmation.key}), {})
        self.assertRedirects(
            response,
            settings.ACCOUNT_LOGIN_REDIRECT_URL,
            fetch_redirect_response=False
        )

    @override_settings(
        ACCOUNT_EMAIL_CONFIRMATION_REQUIRED=False, ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL="/somewhere/"
    )
    def test_post_not_required_redirect_override(self):
        email_confirmation = self.signup()
        response = self.client.post(reverse("account_confirm_email", kwargs={"key": email_confirmation.key}), {})
        self.assertRedirects(
            response,
            settings.ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL,
            fetch_redirect_response=False
        )

    @override_settings(
        ACCOUNT_EMAIL_CONFIRMATION_REQUIRED=True,
        ACCOUNT_EMAIL_CONFIRMATION_AUTO_LOGIN=True,
        ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL="/somewhere/",
    )
    def test_post_auto_login(self):
        email_confirmation = self.signup()
        response = self.client.post(reverse("account_confirm_email", kwargs={"key": email_confirmation.key}), {})
        self.assertRedirects(
            response,
            settings.ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL,
            fetch_redirect_response=False
        )


class ChangePasswordViewTestCase(TestCase):

    def signup(self):
        data = {
            "username": "foo",
            "password": "bar",
            "password_confirm": "bar",
            "email": "foobar@example.com",
            "code": "abc123",
        }
        self.client.post(reverse("account_signup"), data)
        mail.outbox = []
        return User.objects.get(username="foo")

    def test_get_anonymous(self):
        response = self.client.get(reverse("account_password"))
        self.assertRedirects(
            response,
            reverse("account_password_reset"),
            fetch_redirect_response=False
        )

    def test_get_authenticated(self):
        self.signup()
        response = self.client.get(reverse("account_password"))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name, ["account/password_change.html"])

    def test_post_anonymous(self):
        data = {
            "password_current": "password",
            "password_new": "new-password",
            "password_new_confirm": "new-password",
        }
        response = self.client.post(reverse("account_password"), data)
        self.assertEqual(response.status_code, 403)

    def test_post_authenticated_success(self):
        user = self.signup()
        data = {
            "password_current": "bar",
            "password_new": "new-bar",
            "password_new_confirm": "new-bar",
        }
        response = self.client.post(reverse("account_password"), data)
        self.assertRedirects(
            response,
            reverse(settings.ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL),
            fetch_redirect_response=False
        )
        updated_user = User.objects.get(username=user.username)
        self.assertNotEqual(user.password, updated_user.password)
        self.assertEqual(len(mail.outbox), 1)

    @override_settings(ACCOUNT_NOTIFY_ON_PASSWORD_CHANGE=False)
    def test_post_authenticated_success_no_mail(self):
        self.signup()
        data = {
            "password_current": "bar",
            "password_new": "new-bar",
            "password_new_confirm": "new-bar",
        }
        response = self.client.post(reverse("account_password"), data)
        self.assertRedirects(
            response,
            reverse(settings.ACCOUNT_PASSWORD_CHANGE_REDIRECT_URL),
            fetch_redirect_response=False
        )
        self.assertEqual(len(mail.outbox), 0)


class PasswordResetTokenViewTestCase(TestCase):

    def signup(self):
        data = {
            "username": "foo",
            "password": "bar",
            "password_confirm": "bar",
            "email": "foobar@example.com",
            "code": "abc123",
        }
        self.client.post(reverse("account_signup"), data)
        mail.outbox = []
        return User.objects.get(username="foo")

    def request_password_reset(self):
        user = self.signup()
        data = {
            "email": user.email,
        }
        self.client.post(reverse("account_password_reset"), data)
        parsed = urlparse(mail.outbox[0].body.strip())
        return user, parsed.path

    def test_get_bad_user(self):
        url = reverse(
            "account_password_reset_token",
            kwargs={
                "uidb36": int_to_base36(100),
                "token": "notoken",
            }
        )
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_get_abuse_reset_token(self):
        user = self.signup()
        url = reverse(
            "account_password_reset_token",
            kwargs={
                "uidb36": int_to_base36(user.id),
                "token": INTERNAL_RESET_URL_TOKEN,
            }
        )
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response,
                                PasswordResetTokenView.template_name_fail)

    def test_get_reset(self):
        user, url = self.request_password_reset()
        response = self.client.get(url)
        self.assertRedirects(
            response,
            reverse(
                "account_password_reset_token",
                kwargs={
                    "uidb36": int_to_base36(user.id),
                    "token": INTERNAL_RESET_URL_TOKEN,
                }
            ),
            fetch_redirect_response=False
        )

    def test_post_reset(self):
        user, url = self.request_password_reset()
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        data = {
            "password": "new-password",
            "password_confirm": "new-password",
        }
        response = self.client.post(response["Location"], data)
        self.assertRedirects(
            response,
            reverse(settings.ACCOUNT_PASSWORD_RESET_REDIRECT_URL),
            fetch_redirect_response=False
        )



from django.conf import settings
from django.utils.translation import get_language_info

"""
# List of language code and languages local names.
#
# This list is output of code:

[
    (code, get_language_info(code).get("name_local"))
    for code, lang in settings.LANGUAGES
]
"""

LANGUAGES = [
    ("af", "Afrikaans"),
    ("ar", "العربيّة"),
    ("ast", "asturian"),
    ("az", "Azərbaycanca"),
    ("bg", "български"),
    ("be", "беларуская"),
    ("bn", "বাংলা"),
    ("br", "brezhoneg"),
    ("bs", "bosanski"),
    ("ca", "català"),
    ("cs", "česky"),
    ("cy", "Cymraeg"),
    ("da", "dansk"),
    ("de", "Deutsch"),
    ("el", "Ελληνικά"),
    ("en", "English"),
    ("en-au", "Australian English"),
    ("en-gb", "British English"),
    ("eo", "Esperanto"),
    ("es", "español"),
    ("es-ar", "español de Argentina"),
    ("es-mx", "español de Mexico"),
    ("es-ni", "español de Nicaragua"),
    ("es-ve", "español de Venezuela"),
    ("et", "eesti"),
    ("eu", "Basque"),
    ("fa", "فارسی"),
    ("fi", "suomi"),
    ("fr", "français"),
    ("fy", "frysk"),
    ("ga", "Gaeilge"),
    ("gl", "galego"),
    ("he", "עברית"),
    ("hi", "Hindi"),
    ("hr", "Hrvatski"),
    ("hu", "Magyar"),
    ("ia", "Interlingua"),
    ("id", "Bahasa Indonesia"),
    ("io", "ido"),
    ("is", "Íslenska"),
    ("it", "italiano"),
    ("ja", "日本語"),
    ("ka", "ქართული"),
    ("kk", "Қазақ"),
    ("km", "Khmer"),
    ("kn", "Kannada"),
    ("ko", "한국어"),
    ("lb", "Lëtzebuergesch"),
    ("lt", "Lietuviškai"),
    ("lv", "latvieš"),
    ("mk", "Македонски"),
    ("ml", "Malayalam"),
    ("mn", "Mongolian"),
    ("mr", "मराठी"),
    ("my", "မြန်မာဘာသာ"),
    ("nb", "norsk (bokmål)"),
    ("ne", "नेपाली"),
    ("nl", "Nederlands"),
    ("nn", "norsk (nynorsk)"),
    ("os", "Ирон"),
    ("pa", "Punjabi"),
    ("pl", "polski"),
    ("pt", "Português"),
    ("pt-br", "Português Brasileiro"),
    ("ro", "Română"),
    ("ru", "Русский"),
    ("sk", "slovenský"),
    ("sl", "Slovenščina"),
    ("sq", "shqip"),
    ("sr", "српски"),
    ("sr-latn", "srpski (latinica)"),
    ("sv", "svenska"),
    ("sw", "Kiswahili"),
    ("ta", "தமிழ்"),
    ("te", "తెలుగు"),
    ("th", "ภาษาไทย"),
    ("tr", "Türkçe"),
    ("tt", "Татарча"),
    ("udm", "Удмурт"),
    ("uk", "Українська"),
    ("ur", "اردو"),
    ("vi", "Tiếng Việt"),
    ("zh-cn", "简体中文"),
    ("zh-hans", "简体中文"),
    ("zh-hant", "繁體中文"),
    ("zh-tw", "繁體中文")
]

DEFAULT_LANGUAGE = get_language_info(settings.LANGUAGE_CODE)["code"]
